import requests
from bs4 import BeautifulSoup
from fp.fp import FreeProxy

import re
import numpy as np
import pandas as pd


class WebScraper:
    __headers__ = {'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) '
                             'Chrome/87.0.4280.88 Safari/537.36'}

    @staticmethod
    def get_proxy():
        while 1:
            proxy = FreeProxy(country_id=['US'], rand=True).get()
            if proxy is not None:
                break
        return proxy

    @staticmethod
    def get_soup(main_url, use_proxy=False, remove_imports=True):
        if use_proxy:
            p = WebScraper.get_proxy()
            response = requests.get(main_url, headers=WebScraper.__headers__, proxies={'http': p, 'https': p})
        else:
            response = requests.get(main_url, headers=WebScraper.__headers__)

        soup = BeautifulSoup(response.content, 'lxml')
        if remove_imports:
            for script in soup(["script", "style"]):  # remove all javascript and stylesheet code
                script.extract()
        return soup

    @staticmethod
    def get_json(url):
        return requests.get(url, headers=WebScraper.__headers__).json()

    '''
    SOME PAGES ONLY HAVE 1 TABLE PER PAGE.
    FIND THAT TABLE AND RETURN IT AS A pd.DataFrame
    '''

    @staticmethod
    def get_single_table_pandas(main_url):
        soup = WebScraper.get_soup(main_url)
        main_table = soup.find("table", {'class': 'styled-table-new'})
        if main_table is None:
            return soup, pd.DataFrame()
        main_table_rows = main_table.find_all("tr")
        table_header = [re.sub(r'[^a-zA-Z0-9]', '', td.text.strip()) for td in
                        main_table_rows[0].find_all("th", recursive=False)]

        table_info_arr = [[td.text.strip() for td in row.find_all("td", recursive=False)] for row in
                          main_table_rows[1:]]
        table_info_arr = [row for row in table_info_arr if len(row) == len(table_header)]  # removed unmatched rows
        table_info_array = np.asarray(table_info_arr)

        return soup, pd.DataFrame(table_info_array, columns=table_header)



import pandas as pd

from pyfinviz.utils import WebScraper


class News:

    @staticmethod
    def __table_to_df__(table):
        trs = table.find_all('tr', recursive=False)
        info = []

        for tr in trs:
            tds = tr.find_all('td')
            td_a = tds[len(tds)-1].find('a')
            if td_a is None:
                continue
            time = '' if len(tds) < 2 else tds[1].text
            info.append({'Time': time, 'Headline': td_a.text, 'URL': td_a['href']})

        return pd.DataFrame(info)

    def __init__(self):
        self.main_url = 'https://finviz.com/news.ashx'
        self.soup = WebScraper.get_soup(self.main_url)

        div_ = self.soup.find('div', class_='news').find('table')
        trs_ = div_.find_all('tr', recursive=False)
        main_tables = trs_[len(trs_)-1].find_all('table')

        self.news_df = News.__table_to_df__(main_tables[0])
        self.blogs_df = News.__table_to_df__(main_tables[1])



from pyfinviz.utils import WebScraper
import enum


class Crypto:
    class RelativePerformanceOption(enum.Enum):
        ONE_DAY = "1"
        ONE_WEEK = "2"
        ONE_MONTH = "3"
        MONTH_TO_DATE = "8"
        THREE_MONTH = "4"
        HALF_YEAR = "5"
        ONE_YEAR = "6"
        YEAR_TO_DATE = "7"

    def __init__(self, relative_performance_option: RelativePerformanceOption = RelativePerformanceOption.ONE_DAY):
        self.main_url = f'https://finviz.com/crypto_performance.ashx?v={relative_performance_option.value}'
        self.soup, self.table_df = WebScraper.get_single_table_pandas(self.main_url)



from pyfinviz.utils import WebScraper
from enum import Enum


class Groups:
    class OrderBy(Enum):
        NAME = "name"
        MARKET_CAPITALIZATION = "marketcap"
        PRICE_EARNINGS = "pe"
        FORWARD_PRICE_EARNINGS = "forwardpe"
        PEG_PRICE_EARNINGS_GROWTH = "peg"
        PRICE_SALES = "ps"
        PRICE_BOOK = "pb"
        PRICE_CASH = "pc"
        PRICE_FREE_CASH_FLOW = "pfcf"
        DIVIDEND_YIELD = "dividendyield"
        EPS_GROWTH_PAST_5_YEARS = "eps5years"
        EPS_GROWTH_NEXT_5_YEARS = "estltgrowth"
        SALES_GROWTH_PAST_5_YEARS = "sales5years"
        SHORT_INTEREST_SHARE = "shortinterestshare"
        ANALYST_RECOMMENDATION = "recom"
        PERFORMANCE_WEEK = "perf1w"
        PERFORMANCE_MONTH = "perf4w"
        PERFORMANCE_QUARTER = "perf13w"
        PERFORMANCE_HALF_YEAR = "perf26w"
        PERFORMANCE_YEAR = "perf52w"
        PERFORMANCE_YEAR_TO_DATE = "perfytd"
        AVERAGE_VOLUME_3_MONTH = "averagevolume"
        RELATIVE_VOLUME = "relativevolume"
        CHANGE = "change"
        VOLUME = "volume"
        NUMBER_OF_STOCKS = "count"

    class OrderDirection(Enum):
        ASC = ""
        DESC = "-"

    class GroupOption(Enum):
        SECTOR = 'g=sector'
        INDUSTRY = 'g=industry'
        INDUSTRY_BASIC_MATERIALS = f'{INDUSTRY}&sg=basicmaterials'
        INDUSTRY_COMMUNICATION_SERVICES = f'{INDUSTRY}&sg=communicationservices'
        INDUSTRY_CONSUMER_CYCLICAL = f'{INDUSTRY}&sg=consumercyclical'
        INDUSTRY_CONSUMER_DEFENSIVE = f'{INDUSTRY}&sg=consumerdefensive'
        INDUSTRY_ENERGY = f'{INDUSTRY}&sg=energy'
        INDUSTRY_FINANCIAL = f'{INDUSTRY}&sg=financial'
        INDUSTRY_HEALTHCARE = f'{INDUSTRY}&sg=healthcare'
        INDUSTRY_INDUSTRIALS = f'{INDUSTRY}&sg=industrials'
        INDUSTRY_REAL_ESTATE = f'{INDUSTRY}&sg=realestate'
        INDUSTRY_TECHNOLOGY = f'{INDUSTRY}&sg=technology'
        INDUSTRY_UTILITIES = f'{INDUSTRY}&sg=utilities'
        COUNTRY = 'g=country'
        CAPITALIZATION = 'g=capitalization'

    class ViewOption(Enum):
        OVERVIEW = "110"
        VALUATION = "120"
        PERFORMANCE = "140"
        CUSTOM = "150"

    def __init__(self, group_option: GroupOption = GroupOption.SECTOR,
                 view_option: ViewOption = ViewOption.OVERVIEW,
                 order_by: OrderBy = OrderBy.NAME, order_direction: OrderDirection = OrderDirection.ASC):
        order_str = order_direction.value + order_by.value
        self.main_url = f'https://finviz.com/groups.ashx?{group_option.value}&v={view_option.value}&o={order_str}'
        self.soup, self.table_df = WebScraper.get_single_table_pandas(self.main_url)
